#include "jannex/jannex.h"

#include "jannex/otaAPI.h"
#include "jannex/otaUpdate.h"

#include "jannex/httpsRequest.h"
#include "json_parser.h"

#include "esp_system.h"

#include "esp_mac.h"

#include "esp_app_desc.h"
#include "esp_ota_ops.h"

bool updateAvailable = false;
bool isCheckingForUpdates = false;

void getNewestVersion()
{
    isCheckingForUpdates = true;

    // buffer for the response
    size_t length = 256;

    char response[length];
    int status_code = 0;
    int responseLength = 0;

    // get mac address
    uint8_t mac[6];
    esp_efuse_mac_get_default(mac);

    // get current version
    const esp_app_desc_t *app_desc = esp_app_get_description();
    char *currentVersion = app_desc->version;

    char branch[32];
    getBranch(branch);

    char url[256];
    sprintf(url, "%s?mac=%02X:%02X:%02X:%02X:%02X:%02X&device=%s&branch=%s", OTA_URL_PATH_getNewestVersion, mac[0], mac[1], mac[2], mac[3], mac[4], mac[5], app_desc->project_name, branch);

    // make the request
    esp_err_t err = https_get_request_SYNC(url, response, &responseLength, length, &status_code);

    // ESP_LOGI(jannexTAG, "HTTP GET Response: %i %i %s", status_code, responseLength, response);

    jparse_ctx_t jctx;
    int ret = json_parse_start(&jctx, response, responseLength);
    if (ret != OS_SUCCESS)
    {
        ESP_LOGE(jannexTAG, "HTTP GET Response: %i %i %s", status_code, responseLength, response);
        ESP_LOGE(jannexTAG, "Parser failed");
        isCheckingForUpdates = false;
        return;
    }
    char newestVersion[32];

    if (json_obj_get_string(&jctx, "error", newestVersion, sizeof(newestVersion)) == OS_SUCCESS)
    {
        ESP_LOGW(jannexTAG, "Error: %s", newestVersion);
        // ERROR
        isCheckingForUpdates = false;
        return;
    }

    if (json_obj_get_string(&jctx, "version", newestVersion, sizeof(newestVersion)) == OS_SUCCESS)
    {
        // ESP_LOGI(jannexTAG, "newestVersion %s", newestVersion);
    }
    else
    {
        ESP_LOGI(jannexTAG, "No version");
        isCheckingForUpdates = false;
        return;
    }

    // print current version
    // ESP_LOGI(jannexTAG, "Current version: %s", currentVersion);

    // newestVersion and currentVersion are strings with the format two numbers separated by a dot (e.g. 1.2) now we need to compare them
    // first we need to split the strings into two numbers
    char newestVersionMajor[16];
    char newestVersionMinor[16];
    char currentVersionMajor[16];
    char currentVersionMinor[16];

    // split newestVersion
    int i = 0;
    while (newestVersion[i] != '.')
    {
        newestVersionMajor[i] = newestVersion[i];
        i++;
    }
    newestVersionMajor[i] = '\0';
    i++;
    int j = 0;
    while (newestVersion[i] != '\0')
    {
        newestVersionMinor[j] = newestVersion[i];
        i++;
        j++;
    }
    newestVersionMinor[j] = '\0';

    // split currentVersion
    i = 0;
    while (currentVersion[i] != '.')
    {
        currentVersionMajor[i] = currentVersion[i];
        i++;
    }
    currentVersionMajor[i] = '\0';
    i++;
    j = 0;
    while (currentVersion[i] != '\0')
    {
        currentVersionMinor[j] = currentVersion[i];
        i++;
        j++;
    }
    currentVersionMinor[j] = '\0';

    // compare the numbers
    int newestVersionMajorInt = atoi(newestVersionMajor);
    int newestVersionMinorInt = atoi(newestVersionMinor);
    int currentVersionMajorInt = atoi(currentVersionMajor);
    int currentVersionMinorInt = atoi(currentVersionMinor);

    if (newestVersionMajorInt != currentVersionMajorInt) // or only for new version: newestVersionMajorInt > currentVersionMajorInt
    {
        // major update
        ESP_LOGI(jannexTAG, "Major update available");
        updateAvailable = true;
    }
    else if (newestVersionMinorInt != currentVersionMinorInt) // or only for new version: newestVersionMinorInt > currentVersionMinorInt
    {
        ESP_LOGI(jannexTAG, "Minor update available");
        updateAvailable = true;
    }

    isCheckingForUpdates = false;

    // esp ota do update
    if (updateAvailable)
    {

        startOTAupdate();
    }
}

void checkForUpdatesAndInstall(void *parameter)
{
    while (1)
    {
        if (isCheckingForUpdates == false)
            getNewestVersion();

#ifdef CONFIG_FIRMWARE_UPDATE_SCAN_SECONDS
        if (CONFIG_FIRMWARE_UPDATE_SCAN_SECONDS >= 1)
        {
            vTaskDelay(pdMS_TO_TICKS(CONFIG_FIRMWARE_UPDATE_SCAN_SECONDS * 1000)); // Delay for 1000 milliseconds
        }
        else
#endif
        {
            vTaskDelete(NULL);
        }
    }
}
